"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = exports.Pirsch = exports.PirschNodeApiClient = void 0;
const node_url_1 = require("node:url");
const axios_1 = __importStar(require("axios"));
const core_1 = require("./core");
const common_1 = require("./common");
const constants_1 = require("./constants");
/**
 * Client is used to access the Pirsch API.
 */
class PirschNodeApiClient extends core_1.PirschCoreClient {
    /**
     * The constructor creates a new client.
     *
     * @param {object} configuration You need to pass in the **Hostname**, **Client ID** and **Client Secret** or **Access Key** you have configured on the Pirsch dashboard.
     * It's also recommended to set the proper protocol for your website, else it will be set to `https` by default.
     * All other configuration parameters can be left to their defaults.
     * @param {string} configuration.baseUrl The base URL for the pirsch API
     * @param {number} configuration.timeout The default HTTP timeout in milliseconds
     * @param {string} configuration.clientId The OAuth client ID
     * @param {string} configuration.clientSecret The OAuth client secret
     * @param {string} configuration.hostname The hostname of the domain to track
     * @param {string} configuration.protocol The default HTTP protocol to use for tracking
     *
     */
    constructor(configuration) {
        super(configuration);
        const { protocol = constants_1.PIRSCH_DEFAULT_PROTOCOL, hostname, trustedProxyHeaders } = configuration;
        this.hostname = hostname;
        this.protocol = protocol;
        this.trustedProxyHeaders = trustedProxyHeaders;
        this.httpClient = axios_1.default.create({ baseURL: this.baseUrl, timeout: this.timeout });
    }
    /**
     * hitFromRequest returns the required data to send a hit to Pirsch for a Node request object.
     *
     * @param request the Node request object from the http package.
     * @returns Hit object containing all necessary fields.
     */
    hitFromRequest(request) {
        var _a, _b, _c;
        const url = new node_url_1.URL((_a = request.url) !== null && _a !== void 0 ? _a : "", `${this.protocol}://${this.hostname}`);
        const element = {
            url: url.toString(),
            ip: (_b = request.socket.remoteAddress) !== null && _b !== void 0 ? _b : "",
            dnt: this.getHeader(request.headers, "dnt"),
            user_agent: (_c = this.getHeader(request.headers, "user-agent")) !== null && _c !== void 0 ? _c : "",
            accept_language: this.getHeader(request.headers, "accept-language"),
            sec_ch_ua: this.getHeader(request.headers, "Sec-CH-UA"),
            sec_ch_ua_mobile: this.getHeader(request.headers, "Sec-CH-UA-Mobile"),
            sec_ch_ua_platform: this.getHeader(request.headers, "Sec-CH-UA-Platform"),
            sec_ch_ua_platform_version: this.getHeader(request.headers, "Sec-CH-UA-Platform-Version"),
            sec_ch_width: this.getHeader(request.headers, "Sec-CH-Width"),
            sec_ch_viewport_width: this.getHeader(request.headers, "Sec-CH-Viewport-Width"),
            referrer: this.getReferrer(request, url),
        };
        if (this.trustedProxyHeaders && this.trustedProxyHeaders.length > 0) {
            const header = this.trustedProxyHeaders
                .filter(header => {
                return constants_1.PIRSCH_PROXY_HEADERS.includes(header);
            })
                .find(header => {
                return typeof request.headers[header] === "string";
            });
            if (header) {
                const result = this.getHeader(request.headers, header);
                if (result) {
                    element.ip = result;
                }
            }
        }
        return element;
    }
    getReferrer(request, url) {
        var _a, _b;
        const referrer = (_b = (_a = this.getHeader(request.headers, "referer")) !== null && _a !== void 0 ? _a : this.getHeader(request.headers, "referrer")) !== null && _b !== void 0 ? _b : "";
        if (referrer === "") {
            for (const parameterName of constants_1.PIRSCH_REFERRER_QUERY_PARAMETERS) {
                const parameter = url.searchParams.get(parameterName);
                if (parameter && parameter !== "") {
                    return parameter;
                }
            }
        }
        return referrer;
    }
    getHeader(headers, name) {
        const header = headers[name];
        if (Array.isArray(header)) {
            return header.at(0);
        }
        return header;
    }
    get(url, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.httpClient.get(url, this.httpOptionsToAxiosOptions(options));
            return result.data;
        });
    }
    post(url, data, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.httpClient.post(url, data, this.httpOptionsToAxiosOptions(options));
            return result.data;
        });
    }
    toApiError(error) {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            if (error instanceof common_1.PirschApiError) {
                return error;
            }
            if (error instanceof axios_1.AxiosError && error.response !== undefined) {
                const exception = error;
                return new common_1.PirschApiError((_b = (_a = exception.response) === null || _a === void 0 ? void 0 : _a.status) !== null && _b !== void 0 ? _b : 500, (_d = (_c = exception.response) === null || _c === void 0 ? void 0 : _c.data) !== null && _d !== void 0 ? _d : { validation: {}, error: [] });
            }
            if (typeof error === 'object' && error !== null && 'response' in error && typeof error.response === 'object' && error.response !== null && 'status' in error.response && 'data' in error.response) {
                return new common_1.PirschApiError((_e = error.response.status) !== null && _e !== void 0 ? _e : 400, (_f = error.response) === null || _f === void 0 ? void 0 : _f.data);
            }
            if (error instanceof Error) {
                return new common_1.PirschUnknownApiError(error.message);
            }
            if (typeof error === 'object' && error !== null && 'message' in error && typeof error.message === 'string') {
                return new common_1.PirschUnknownApiError(error.message);
            }
            return new common_1.PirschUnknownApiError(JSON.stringify(error));
        });
    }
    httpOptionsToAxiosOptions(options) {
        const result = {};
        if (options === null || options === void 0 ? void 0 : options.headers) {
            result.headers = options.headers;
        }
        if (options === null || options === void 0 ? void 0 : options.parameters) {
            result.params = options.parameters;
        }
        return result;
    }
}
exports.PirschNodeApiClient = PirschNodeApiClient;
exports.Pirsch = PirschNodeApiClient;
exports.Client = PirschNodeApiClient;
