import { PirschTokenClientConfig, PirschHttpOptions } from "./types";
import { PirschCoreClient } from "./core";
import { PirschApiError } from "./common";
/**
 * Client is used to access the Pirsch API.
 */
export declare class PirschWebApiClient extends PirschCoreClient {
    private httpClient;
    /**
     * The constructor creates a new client.
     *
     * @param {object} configuration You need to pass in the **Access Token** you have configured on the Pirsch dashboard.
     * It's also recommended to set the proper protocol for your website, else it will be set to `https` by default.
     * All other configuration parameters can be left to their defaults.
     * @param {string} configuration.baseUrl The base URL for the pirsch API
     * @param {number} configuration.timeout The default HTTP timeout in milliseconds
     * @param {string} configuration.accessToken The access token
     *
     */
    constructor(configuration: PirschTokenClientConfig);
    protected get<Response>(url: string, options?: PirschHttpOptions): Promise<Response>;
    protected post<Response, Data extends object = object>(url: string, data: Data, options?: PirschHttpOptions): Promise<Response>;
    protected toApiError(error: unknown): Promise<PirschApiError>;
    private createOptions;
}
export declare const Pirsch: typeof PirschWebApiClient;
export declare const Client: typeof PirschWebApiClient;
