"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = exports.Pirsch = exports.PirschWebApiClient = void 0;
const axios_1 = __importStar(require("axios"));
const core_1 = require("./core");
const common_1 = require("./common");
/**
 * Client is used to access the Pirsch API.
 */
class PirschWebApiClient extends core_1.PirschCoreClient {
    /**
     * The constructor creates a new client.
     *
     * @param {object} configuration You need to pass in the **Access Token** you have configured on the Pirsch dashboard.
     * It's also recommended to set the proper protocol for your website, else it will be set to `https` by default.
     * All other configuration parameters can be left to their defaults.
     * @param {string} configuration.baseUrl The base URL for the pirsch API
     * @param {number} configuration.timeout The default HTTP timeout in milliseconds
     * @param {string} configuration.accessToken The access token
     *
     */
    constructor(configuration) {
        if ("clientId" in configuration || "clientSecret" in configuration) {
            throw new Error("Do not pass OAuth secrets such as 'clientId' or 'clientSecret' to the web client!");
        }
        super(configuration);
        this.httpClient = axios_1.default.create({
            baseURL: this.baseUrl,
            timeout: this.timeout,
        });
    }
    get(url, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.httpClient.get(url, this.createOptions(Object.assign({}, options)));
            return result.data;
        });
    }
    post(url, data, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.httpClient.post(url, data, this.createOptions(Object.assign(Object.assign({}, options), { data })));
            return result.data;
        });
    }
    toApiError(error) {
        var _a, _b, _c, _d, _e;
        return __awaiter(this, void 0, void 0, function* () {
            if (error instanceof common_1.PirschApiError) {
                return error;
            }
            if (error instanceof axios_1.AxiosError) {
                return new common_1.PirschApiError((_b = (_a = error.response) === null || _a === void 0 ? void 0 : _a.status) !== null && _b !== void 0 ? _b : 400, (_c = error.response) === null || _c === void 0 ? void 0 : _c.data);
            }
            if (typeof error === 'object' && error !== null && 'response' in error && typeof error.response === 'object' && error.response !== null && 'status' in error.response && 'data' in error.response) {
                return new common_1.PirschApiError((_d = error.response.status) !== null && _d !== void 0 ? _d : 400, (_e = error.response) === null || _e === void 0 ? void 0 : _e.data);
            }
            if (error instanceof Error) {
                return new common_1.PirschUnknownApiError(error.message);
            }
            if (typeof error === 'object' && error !== null && 'message' in error && typeof error.message === 'string') {
                return new common_1.PirschUnknownApiError(error.message);
            }
            return new common_1.PirschUnknownApiError(JSON.stringify(error));
        });
    }
    createOptions({ headers, parameters, data }) {
        return {
            headers,
            params: parameters,
            data,
        };
    }
}
exports.PirschWebApiClient = PirschWebApiClient;
exports.Pirsch = PirschWebApiClient;
exports.Client = PirschWebApiClient;
